//CA.cpp              //If you don't mind doing command line compiling with  
#include <windows.h>  //Microsoft C++ 6 you can use this command line...
#include <stdio.h>    //
#include "CA.h"       //cl /LD CA.cpp Server.cpp Registry.cpp UUID.lib Advapi32.lib Ole32.lib CA.def 


CA::CA()                         //CA Constructor.  The creation/instantiation of every 
{                                //object will cause a thread safe increment of the global
 m_lRef=0;                       //count of CA objects created in the Server.  The CA 
 InterlockedIncrement(&g_lObjs); //member variable m_lRef will count the number of outstanding
}                                //interface pointers handed out through QueryInterface().


CA::~CA()
{                                //When a CA object is destroyed the count of objects is  
 InterlockedDecrement(&g_lObjs); //decremented by the thread safe InterlockedDecrement()
}                                //function.


HRESULT __stdcall CA::QueryInterface(REFIID riid, void** ppv)
{
 *ppv=0;                  //Store initial null at address pointed to by ppv.
 if(riid==IID_IUnknown)   //If GUID of either IID_IUnknown or IID_IX
    *ppv=(I_X*)this;      //is passed 'in', cast this pointer to point
 else if(riid==IID_I_X)   //to the 1st of the two pVTbl pointers that constitute
    *ppv=(I_X*)this;      //the class.  If client wants an IID_IY interface, return
 else if(riid==IID_I_Y)   //the 2nd pointer - which pointer points to the IY
    *ppv=(I_Y*)this;      //VTable.  If *ppv has been set, then do an AddRef() and
 if(*ppv)                 //return S_OK.  Otherwise, return E_NOINTERFACE.  The
 {                        //printf statement below will only execute if a non-
    AddRef();             //supported interface IID was passed in through the 
    return S_OK;          //REFIID parameter.  This is how the Called CA::QueryInterface()
 }                        //output was generated in the client app through a function
 printf("Called CA::QueryInterface()\n");  //pointer call.

 return(E_NOINTERFACE);
}


ULONG __stdcall CA::AddRef()            //The member variable of class CA this->m_lRef 
{                                       //counts interface pointers handed out by 
 printf("Called CA::AddRef()\n");       //QueryInterface().  InterlockedIncrement takes
 return InterlockedIncrement(&m_lRef);  //a pointer parameter so the address ( & ) of
}                                       //this->m_lRef is passed in.


ULONG __stdcall CA::Release()           //When you are done using an interface pointer
{                                       //Release() should be called on the pointer so
 printf("Called CA::Release()\n");      //that the count of references to the object can
 if(InterlockedDecrement(&m_lRef)==0)   //be decremented.  When the count reaches zero
 {                                      //the class automatically deletes itself from
    delete this;                        //memory., i.e., delete this;.  However, the 
    return 0;                           //dll server may still reside in memory if the
 }                                      //global count of objects created is greater
                                        //than zero, i.e., g_lObjs isn't zero..  
 return m_lRef;
}


HRESULT __stdcall CA::Fx1(int iNum)             //These functions just output a 
{                                               //message that they were called 
 printf("Called Fx1()  :  iNum = %u\n",iNum);   //plus the value of the parameter
 return S_OK;                                   //passed in.  Fx1() and Fx2() are
}                                               //part of the I_X interface/vtable


HRESULT __stdcall CA::Fx2(int iNum)             //and Fy1() and Fy2() part of the
{                                               //I_Y vtable, i.e., these respective
 printf("Called Fx2()  :  iNum = %u\n",iNum);   //vtables contain pointers to these
 return S_OK;                                   //functions.
}


HRESULT __stdcall CA::Fy1(int iNum)
{
 printf("Called Fy1()  :  iNum = %u\n",iNum);
 return S_OK;
}


HRESULT __stdcall CA::Fy2(int iNum)
{
 printf("Called Fy2()  :  iNum = %u\n",iNum);
 return S_OK;
}


CAClassFactory::CAClassFactory()  //CAClassFactory Constructor.  Four procedures are exported from
{                                 //a com server.  They are DllGetClassObject(), DllCanUnloadNow()
 m_lRef=0;                        //DllRegisterServer(), and DllUnregisterServer().  When a 
}                                 //client app calls CoCreateInstance() with the CLSID of a.... 
                                  

CAClassFactory::~CAClassFactory() //component that can be located in the registry, code internal
{                                 //to COM's implementation will do a LoadLibrary() and GetProcAddress()
 //MathClassFactory Destructor    //on DllGetProcAddress().  See my discussion in Server.cpp about this. 
}


HRESULT __stdcall CAClassFactory::QueryInterface(REFIID riid, void** ppv)
{
 *ppv=0;
 if(riid==IID_IUnknown || riid==IID_IClassFactory)  //In this component that doesn't do very much,
    *ppv=this;                                      //the whole class factory thing seems kind of
 if(*ppv)                                           //superflous.  In other words, when COM Services
 {                                                  //calls DllGetClassObject(), why not just create
    AddRef();                                       //the component CA directly with the C++ new
    return S_OK;                                    //operator instead of 1st creating a class
 }                                                  //factory which in this case does absolutely
                                                    //nothing but create and destroy itself (and  
 return E_NOINTERFACE;                              //uses new to create CA in the process)?  Well,
}                                                   //I suppose you can think of the class factory


ULONG __stdcall CAClassFactory::AddRef()            //concept something like the WM_CREATE message
{                                                   //in Windows Api coding.  In that context one
 return InterlockedIncrement(&m_lRef);              //usually creates the various user interface
}                                                   //elements, i.e., buttons, labels, etc., that


ULONG __stdcall CAClassFactory::Release()           //will appear on the window being created.  That
{                                                   //is how the class factory can be used.  It does
 if(InterlockedDecrement(&m_lRef)==0)               //whatever it takes to uniquely create the object.
 {
    delete this;
    return 0;
 }

 return m_lRef;
}


HRESULT __stdcall CAClassFactory::CreateInstance(LPUNKNOWN pUnkOuter, REFIID riid, void** ppvObj)
{
 HRESULT hr;                           //You won't find anywhere within the code here within any
 CA* pCA;                              //of these project files where CAClassFactory::CreateInstance()
                                       //is called to create a new instance of class CA.  This
 *ppvObj=0;                            //function gets called in a somewhat roundabout manner by
 pCA=new CA;                           //the client app's CoCreateInstance() or CoGetClassObject()
 if(pCA==0)                            //call.  See my discussion in Server.cpp.  First the Windows
    return E_OUTOFMEMORY;              //COM subsystem loads the dll from info it obtains on the
 hr=pCA->QueryInterface(riid,ppvObj);  //class from the registry.  Then it calls DllGetClassObject(),
 if(FAILED(hr))                        //which is an exported function.  Internal Windows code then
    delete pCA;                        //creates a CAClassFactory instance and from that instance
                                       //calls the CAClassFactory::CreateInstance() code you see  
 return hr;                            //here.  And just left you see where a 'new' CA is created,
}                                      //and a QueryInterface() done on it to see if riid is there.


HRESULT __stdcall CAClassFactory::LockServer(BOOL fLock)
{
 if(fLock)                             //This function can be called to lock the dll in memory
    InterlockedIncrement(&g_lLocks);   //even if no outstanding instances of CA are present.  This
 else                                  //works good in cases where you need to create and destroy
    InterlockedDecrement(&g_lLocks);   //objects without having to reload the dll between creation/
                                       //destruction cycles.
 return S_OK;
}
//End CA.cpp
