//Server.cpp            //Note that all the functions in this file are exported.  That's what 
#include <windows.h>    //STDAPI in C mumbo jumbo stands for.  Its a macro that expands to
#include <initguid.h>   //extern "C" HRESULT __export __stdcall. The extern "C" part tells the 
#include "CA.h"         //C++ compiler not to mangle the function names so that other system 
#include "Registry.h"   //code can recognize these names in a GetProcAddress() call.

//Globals
HINSTANCE  g_hModule           = NULL;              //Store dll instance handle
const char g_szFriendlyName[]  = "Com Object CA";   //Store friendly name of component
const char g_szVerIndProgID[]  = "ComObject.CA";    //Store Version Independent ProgID
const char g_szProgID[]        = "ComObject.CA.1";  //Store Versioned Program ID.
long       g_lObjs             = 0;                 //count of ooustanding objects
long       g_lLocks            = 0;                 //used to keep server from unloading


STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, void** ppv)  
{                                    
 CAClassFactory* pCF=NULL;           //DllGetClassObject() is critically important because
 HRESULT hr;                         //it is this function that the operating system calls
                                     //when a client attempts to instantiate an object 
 if(rclsid!=CLSID_CA)                //contained within this dll through a CoCreateInstance()
    return E_FAIL;                   //call, or, in the case of the PowerBASIC compiler when
 pCF=new CAClassFactory;             //it internally makes the call on the client's behalf.
 if(pCF==0)                          //Whether PowerBASIC does it or we do it through a 
    return E_OUTOFMEMORY;            //CoCreateInstance() call, the operating system locates
 hr=pCF->QueryInterface(riid,ppv);   //the path to the dll containing some specific CLSID 
 if(FAILED(hr))                      //from the \CLSID\InProcServer32 key in the registry,
    delete pCF;                      //does a LoadLibrary() and GetProcAddress() call on the
                                     //dll, and then calls DllGetClassObject().  If the CLSID
 return hr;                          //passed in matches CLSID_CA, then the C++ 'new' operator
}                                    //is used to create an instance of CA's Class Factory (in...


STDAPI DllCanUnloadNow()             //CA.cpp.  Having done that it then checks to see if it
{                                    //can obtain an IUnknown or IClassFactory pointer on CA's 
 if(g_lObjs||g_lLocks)               //Class Factory.  If it can the reference count on the 
    return S_FALSE;                  //class factory is incremented.  What happens next is 
 else                                //critical.  COM internal system code now calls 
    return S_OK;                     //CAClassFactory::CreateInstance() and that is what 
}                                    //actually causes a new instance of the CA class to be...


STDAPI DllRegisterServer()           //created - which creation causes memory allocations for
{                                    //both the class's VTable pointers and the VTables
 return RegisterServer               //themselves.  Further, class CA implements the abstract
 (                                   //base classes I_X and I_Y, and pointers to the  
  g_hModule,                         //implementations of the pure virtual functions contained
  CLSID_CA,                          //within these base classes are placed in the respective 
  g_szFriendlyName,                  //VTable of each interface.  At this point we have an   
  g_szVerIndProgID,                  //object in memory which can be called to do work for us.
  g_szProgID
 );                                  //DllRegisterServer() and DllUnregisterServer are for
}                                    //adding or eliminating class ids, program ids, etc.,...


STDAPI DllUnregisterServer()         //relating to the component from the registry.  They are
{                                    //called by the RegSvr32.exe Windows component.  For
 return UnregisterServer             //example, on this development machine CA.dll is in
 (                                   //C:\Code\VStudio\VC++6\Projects\COM\ComObjects\CA\Release\CA.dll
  CLSID_CA,                          //I'd register it by doing this...
  g_szVerIndProgID, 
  g_szProgID                         //RegSvr32 C:\Code\VStudio\VC++6\Projects\COM\ComObjects\CA\Release\CA.dll
 );
}                                    //Having done that I thankfully saw a message box telling...


BOOL APIENTRY DllMain(HINSTANCE hInst, DWORD reason, LPVOID reserved)
{
 switch (reason)                     //me registration was successful.  Registry code is pretty 
 {                                   //horrible and in this code was relagated to Registry.cpp
  case DLL_PROCESS_ATTACH:           //Note that that code needs the dll's instance handle
    g_hModule=hInst;                 //and that is saved just at left in DllMain in a global
    break;                           //variable just for that purpose.
  case DLL_PROCESS_DETACH:
    break;
 }

 return TRUE;                        //Returning FALSE causes the dll to not load.
}
//End Server.cpp
